; bayer.asm, Bayer dithering routine
; Copyright 2004 Chris Williams <cow3.14@juno.com>
;
; This program is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with this program; if not, write to the Free Software
; Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

; This routine tells whether a given pixel is on or off depending on its
; location (row and column) and its grey value.
; It implements Bayer dithering using 4x4 dither patterns.
; It looks up the threshold for the row and column in a table and compares it to
; the supplied grey level. If the level is less than the threshold, then the
; pixel is set, otherwise it's clear;

; input:
; 	d = gray value (0 = black, 255 = white)
; 	b = row, c = column
; output:
; 	carry is set if pixel is on
; 70 cycles
; 34 bytes min, 49 bytes max
bayerdither:
	ld	a,b		; 4
	add	a,a		; 4
	add	a,a		; 4
	and	3<<2		; 7
	ld	b,a		; 4
	
	ld	a,c		; 4
	and	3		; 7
	add	a,b		; 4
	or	.levels&0b11110000	; 7
	ld	l,a		; 4
	ld	h,.levels>>8	; 7
	
	ld	a,d		; 4
	cp	(hl)		; 7
	ret			; 10

; These are the minimum grey levels for which each pixel is off (white)
; To change the dither pattern, use each of the numbers 0 to 15, then multiply
; by 16 and add 8 to them.
; The lowest numbers go white first, or in other words, the highest numbers go
; black first.
 align 16
.levels
	db	152, 88,168,104
	db	 40,248,  8,216
	db	184,120,136, 72
	db	 24,200, 56,232
